package ffp_examples;

/*
 *  checker.java
 *  This program demonstrates texture mapping and texture
 *  filtering by mapping a checkerboard image onto six
 *  rectangles. From left to right on both rows, the rectangles have the
 *  following filters: nearest index, dithered, bilinear.
 */
public class checker extends DemoBase {

	public static void main(String[] args) {
		new checker().init(EUT_RGB);
	}

	static int checkImageWidth = 64;
	static int checkImageHeight = 64;
	static int[] checkImage = new int[checkImageWidth * checkImageHeight];

	static void gen_check_image() {
		int i, j;

		for (i = 0; i < checkImageWidth; i++) {
			for (j = 0; j < checkImageHeight; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					checkImage[i + j * checkImageWidth] = 0xffffff;
				} else {
					checkImage[i + j * checkImageWidth] = 0;
				}
			}
		}
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 30.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0.0f, 0.0f, -3.6f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_FLAT);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		gen_check_image();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, checkImage, checkImageWidth, checkImageHeight);
		ffp.ffpEnable(FFP_TEXTURE_2D);
	}

	@Override
	public void dbUpdate() {

	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpScalef(0.5f, 0.5f, 0.5f);
		{
			ffp.ffpPushMatrix();
			ffp.ffpTranslatef(-2, 2, 0);
			ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex3f(-2.0f, -1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 0.25f);
			ffp.ffpVertex3f(-2.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.25f);
			ffp.ffpVertex3f(0.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.0f);
			ffp.ffpVertex3f(0.0f, -1.0f, 0.0f);
			ffp.ffpEnd();
			ffp.ffpTranslatef(-3, -4, 0);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex3f(1.0f, -1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 0.25f);
			ffp.ffpVertex3f(1.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.25f);
			ffp.ffpVertex3f(2.41421f, 1.0f, -1.41421f);
			ffp.ffpTexCoord2f(0.25f, 0.0f);
			ffp.ffpVertex3f(2.41421f, -1.0f, -1.41421f);
			ffp.ffpEnd();
			ffp.ffpPopMatrix();
		}
		{
			ffp.ffpPushMatrix();
			ffp.ffpTranslatef(1, 2, 0);
			ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_DITHER);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex3f(-2.0f, -1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 0.25f);
			ffp.ffpVertex3f(-2.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.25f);
			ffp.ffpVertex3f(0.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.0f);
			ffp.ffpVertex3f(0.0f, -1.0f, 0.0f);
			ffp.ffpEnd();
			ffp.ffpTranslatef(-3, -4, 0);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex3f(1.0f, -1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 0.25f);
			ffp.ffpVertex3f(1.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.25f);
			ffp.ffpVertex3f(2.41421f, 1.0f, -1.41421f);
			ffp.ffpTexCoord2f(0.25f, 0.0f);
			ffp.ffpVertex3f(2.41421f, -1.0f, -1.41421f);
			ffp.ffpEnd();
			ffp.ffpPopMatrix();
		}
		{
			ffp.ffpPushMatrix();
			ffp.ffpTranslatef(4, 2, 0);
			ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_BILINEAR);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex3f(-2.0f, -1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 0.25f);
			ffp.ffpVertex3f(-2.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.25f);
			ffp.ffpVertex3f(0.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.0f);
			ffp.ffpVertex3f(0.0f, -1.0f, 0.0f);
			ffp.ffpEnd();
			ffp.ffpTranslatef(-3, -4, 0);
			ffp.ffpBegin(FFP_QUADS);
			ffp.ffpTexCoord2f(0.0f, 0.0f);
			ffp.ffpVertex3f(1.0f, -1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.0f, 0.25f);
			ffp.ffpVertex3f(1.0f, 1.0f, 0.0f);
			ffp.ffpTexCoord2f(0.25f, 0.25f);
			ffp.ffpVertex3f(2.41421f, 1.0f, -1.41421f);
			ffp.ffpTexCoord2f(0.25f, 0.0f);
			ffp.ffpVertex3f(2.41421f, -1.0f, -1.41421f);
			ffp.ffpEnd();
			ffp.ffpPopMatrix();
		}
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
