package ffp_examples;

public class dof extends DemoBase implements jitter_h {

	public static void main(String[] args) {
		new dof().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
	}

	static final float PI_ = 3.1415927f;

	/*	accFrustum()
	 *  The first 6 arguments are identical to the ffp.ffpFrustum() call.
	 *  
	 *  pixdx and pixdy are anti-alias jitter in pixels. 
	 *  Set both equal to 0.0 for no anti-alias jitter.
	 *  eyedx and eyedy are depth-of field jitter in pixels. 
	 *  Set both equal to 0.0 for no depth of field effects.
	 *
	 *  focus is distance from eye to plane in focus. 
	 *  focus must be greater than, but not equal to 0.0.
	 *
	 *  Note that accFrustum() calls ffp.ffpTranslatef().  You will 
	 *  probably want to insure that your ModelView matrix has been 
	 *  initialized to identity before calling accFrustum().
	 */
	void accFrustum(float left, float right, float bottom, float top, float near, float far, float pixdx, float pixdy, float eyedx, float eyedy, float focus) {
		float xwsize, ywsize;
		float dx, dy;

		int[] viewport = new int[4];
		ffp.ffpGetIntegerv(FFP_VIEWPORT, viewport);

		xwsize = right - left;
		ywsize = top - bottom;

		dx = -(pixdx * xwsize / viewport[2] + eyedx * near / focus);
		dy = -(pixdy * ywsize / viewport[3] + eyedy * near / focus);

		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(left + dx, right + dx, bottom + dy, top + dy, near, far);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(-eyedx, -eyedy, 0.0f);
	}

	/*  accPerspective()
	 * 
	 *  The first 4 arguments are identical to the aux.ffpaux.ffpaux.ffpauxPerspective() call.
	 *  pixdx and pixdy are anti-alias jitter in pixels. 
	 *  Set both equal to 0.0 for no anti-alias jitter.
	 *  eyedx and eyedy are depth-of field jitter in pixels. 
	 *  Set both equal to 0.0 for no depth of field effects.
	 *
	 *  focus is distance from eye to plane in focus. 
	 *  focus must be greater than, but not equal to 0.0.
	 *
	 *  Note that accPerspective() calls accFrustum().
	 */
	void accPerspective(float fovy, float aspect, float near, float far, float pixdx, float pixdy, float eyedx, float eyedy, float focus) {
		float fov2, left, right, bottom, top;

		fov2 = ((fovy * PI_) / 180.0f) / 2.0f;

		top = near / (cos(fov2) / sin(fov2));
		bottom = -top;

		right = top * aspect;
		left = -right;

		accFrustum(left, right, bottom, top, near, far, pixdx, pixdy, eyedx, eyedy, focus);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		float ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
		float diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		float position[] = { 0.0f, 3.0f, 3.0f, 0.0f };

		ffp.ffpEnable(FFP_DEPTH_TEST);

		ffp.ffpLightfv(0, FFP_AMBIENT, ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, diffuse);
		ffp.ffpLightfv(0, FFP_POSITION, position);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_NORMALIZE);

		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();

		ffp.ffpClearColor(0.0f, 0.0f, 0.0f);
		ffp.ffpClearAccum(0.0f, 0.0f, 0.0f);
	}

	@Override
	public void dbUpdate() {

	}

	static final int ACSIZE = 8;

	@Override
	public void dbRender() {
		int[] viewport = new int[4];
		int jitter;

		ffp.ffpGetIntegerv(FFP_VIEWPORT, viewport);

		ffp.ffpClear(FFP_ACCUM_BUFFER);
		for (jitter = 0; jitter < ACSIZE; jitter++) {
			ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
			accPerspective(45.0f, (float) viewport[2] / (float) viewport[3], 1.0f, 15.0f, 0.0f, 0.0f, 0.33f * j8[jitter][0], 0.33f * j8[jitter][1], 5.0f);
			/*	ruby, gold, silver, emerald, and cyan tori	*/
			renderTorus(-1.1, -0.5, -4.5, 0.1745, 0.01175, 0.01175, 0.61424, 0.04136, 0.04136, 0.727811, 0.626959, 0.626959, 0.6);
			renderTorus(-0.5, -0.5, -5.0, 0.24725, 0.1995, 0.0745, 0.75164, 0.60648, 0.22648, 0.628281, 0.555802, 0.366065, 0.4);
			renderTorus(0.2, -0.5, -5.5, 0.19225, 0.19225, 0.19225, 0.50754, 0.50754, 0.50754, 0.508273, 0.508273, 0.508273, 0.4);
			renderTorus(1.0, -0.5, -6.0, 0.0215, 0.1745, 0.0215, 0.07568, 0.61424, 0.07568, 0.633, 0.727811, 0.633, 0.6);
			renderTorus(1.8, -0.5, -6.5, 0.0, 0.1, 0.06, 0.0, 0.50980392, 0.50980392, 0.50196078, 0.50196078, 0.50196078, .25);
			ffp.ffpFlush(FFP_MULTI_THREADED);
			ffp.ffpAccum(FFP_ACCUM, 1.0f / ACSIZE);
		}
		ffp.ffpFlush(FFP_MULTI_THREADED);
		ffp.ffpAccum(FFP_RETURN, 1.0f);
	}


	/*  display() draws 5 tri into the accumulation buffer 
	 *  several times; each time with a jittered perspective.
	 *  The focal point is at z = 5.0, so the gold teapot will 
	 *  stay in focus.  The amount of jitter is adjusted by the
	 *  magnitude of the accPerspective() jitter; in this example, 0.33.
	 *  In this example, the tri are drawn 8 times.  See jitter.h
	 */
	void renderTorus(double x, double y, double z, double ambr, double ambg, double ambb, double difr, double difg, double difb, double specr, double specg, double specb, double shine) {
		float[] mat = new float[3];

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef((float) x, (float) y, (float) z);
		double gamma = 1.5;
		// color^2.2 is done because these color values were originally meant for a non gamma-corrected renderer
		mat[0] = pow(ambr, gamma);
		mat[1] = pow(ambg, gamma);
		mat[2] = pow(ambb, gamma);
		ffp.ffpMaterialfv(FFP_AMBIENT, mat);
		mat[0] = pow(difr, gamma);
		mat[1] = pow(difg, gamma);
		mat[2] = pow(difb, gamma);
		ffp.ffpMaterialfv(FFP_DIFFUSE, mat);
		mat[0] = pow(specr, gamma);
		mat[1] = pow(specg, gamma);
		mat[2] = pow(specb, gamma);
		ffp.ffpMaterialfv(FFP_SPECULAR, mat);
		ffp.ffpMaterialf(FFP_SHININESS, (float) shine * 128.0f);
		aux.auxSolidTorus(0.25f, 0.5f);
		ffp.ffpPopMatrix();
	}
}
