package ffp_examples;

/*
 * lines.java
 * This program demonstrates line stippling.
 */
public class lines extends DemoBase {

	public static void main(String[] args) {
		new lines().init(EUT_RGB | EUT_NOT_RESIZABLE);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxOrtho2D(0.0f, w, 0.0f, h);
	}

	@Override
	public void dbInit(int width, int height) {
		create_projection(450, 150);
		ffp.ffpShadeModel(FFP_FLAT);
	}

	@Override
	public void dbUpdate() {

	}

	private static void draw_line(float x1, float y1, float x2, float y2) {
		ffp.ffpBegin(FFP_LINES);
		ffp.ffpVertex2f(x1, y1);
		ffp.ffpVertex2f(x2, y2);
		ffp.ffpEnd();
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER);
		int i;

		/* select white for all lines  */
		ffp.ffpColor1f(1.0f);

		/* in 1st row, 3 lines, each with a different stipple */
		ffp.ffpEnable(FFP_LINE_STIPPLE);

		ffp.ffpLineStipple(1, (short) 0x0101); /* dotted */
		draw_line(50.0f, 125.0f, 150.0f, 125.0f);
		ffp.ffpLineStipple(1, (short) 0x00ff); /* dashed */
		draw_line(150.0f, 125.0f, 250.0f, 125.0f);
		ffp.ffpLineStipple(1, (short) 0x1c47); /* dash/dot/dash */
		draw_line(250.0f, 125.0f, 350.0f, 125.0f);

		/* in 2nd row, 3 width lines, each with different stipple */
		ffp.ffpLineWidth(5);
		ffp.ffpLineStipple(1, (short) 0x0101);
		draw_line(50.0f, 100.0f, 150.0f, 100.0f);
		ffp.ffpLineStipple(1, (short) 0x00ff);
		draw_line(150.0f, 100.0f, 250.0f, 100.0f);
		ffp.ffpLineStipple(1, (short) 0x1c47);
		draw_line(250.0f, 100.0f, 350.0f, 100.0f);
		ffp.ffpLineWidth(1);

		/* in 3rd row, 6 lines, with dash/dot/dash stipple, */
		/* as part of a single connected line strip */
		ffp.ffpLineStipple(1, (short) 0x1c47); /* dash/dot/dash */
		ffp.ffpBegin(FFP_LINE_STRIP);
		for (i = 0; i < 7; i++) {
			ffp.ffpVertex2f(50.0f + (i * 50.0f), 75.0f);
		}
		ffp.ffpEnd();

		/* in 4th row, 6 independent lines, */
		/* with dash/dot/dash stipple */
		for (i = 0; i < 6; i++) {
			draw_line(50.0f + (i * 50.0f), 50.0f, 50.0f + ((i + 1) * 50.0f), 50.0f);
		}

		/* in 5th row, 1 line, with dash/dot/dash stipple */
		/* and repeat factor of 5 */
		ffp.ffpLineStipple(5, (short) 0x1c47); /* dash/dot/dash */
		draw_line(50.0f, 25.0f, 350.0f, 25.0f);

		ffp.ffpDisable(FFP_LINE_STIPPLE);
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
