package ffp_examples;

public class motionblur extends DemoBase implements jitter_h {

	public static void main(String[] args) {
		new motionblur().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
	}

	static final float PI_ = 3.1415927f;
	static final float FRUSTDIM = 100;
	static final float FRUSTNEAR = 320;
	static final float FRUSTFAR = 660;
	static final int TEXDIM = 256;

	int[] make_texture(int w, int h) {
		int[] texture = new int[w * h];

		for (int i = 0; i < w; i++) {
			for (int j = 0; j < h; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					texture[i + j * w] = 0xffffff;
				} else {
					texture[i + j * w] = 0;
				}
			}
		}
		return texture;
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);

		/* draw a perspective scene */
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpFrustum(-FRUSTDIM, FRUSTDIM, -FRUSTDIM, FRUSTDIM, FRUSTNEAR, FRUSTFAR);
		ffp.ffpMatrixMode(FFP_MODELVIEW);

		/* turn on features */
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);

		/* place light 0 in the right place */
		ffp.ffpLightfv(0, FFP_POSITION, lightpos);

		/* remove back faces to speed things up */
		ffp.ffpCullFace(FFP_BACK);

		int[] tex = make_texture(TEXDIM, TEXDIM);
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, TEXDIM, TEXDIM);
		ffp.ffpClearColor(0.0f, 0.0f, 0.0f);
		ffp.ffpClearAccum(0.0f, 0.0f, 0.0f);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_S)) {
			rendermode = rendermode == NONE ? FIELD : NONE;
		}
	}

	static final int ACSIZE = 8;
	static final int NONE = 0;
	static final int FIELD = 1;
	int rendermode = NONE;

	@Override
	public void dbRender() {
		int i;
		int max;
		float dx, dy, dz;

		dx = .5f;
		dy = 1.f;
		dz = -2.f;

		ffp.ffpPushMatrix();
		switch (rendermode) {
			case NONE:
				render(0.f, 0.f, 0.f);
				ffp.ffpFlush(FFP_MULTI_THREADED);
				break;
			case FIELD:
				max = 16;

				ffp.ffpClear(FFP_ACCUM_BUFFER);

				for (i = 0; i < max; i++) {
					render(dx * i, dy * i, dz * i);
					ffp.ffpFlush(FFP_MULTI_THREADED);
					ffp.ffpAccum(FFP_ACCUM, 1.f / max);
				}
				ffp.ffpFlush(FFP_MULTI_THREADED);
				ffp.ffpAccum(FFP_RETURN, 1.f);
				break;
			default:
				break;
		}

		ffp.ffpPopMatrix();
	}

	static float wall_mat[] = { 1.f, 1.f, 1.f, 1.f };
	static float lightpos[] = { 50.f, 50.f, -320.f, 1.f };
	static float sphere_mat[] = { 1.f, .5f, 0.f, 1.f };
	static float cone_mat[] = { 0.f, .5f, 1.f, 1.f };

	void render(float dx, float dy, float dz) {
		/* material properties for objects in scene */

		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		/*
		** Note: wall verticies are ordered so they are all front facing
		** this lets me do back face culling to speed things up.
		*/

		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, wall_mat);

		/* floor */
		/* make the floor textured */
		ffp.ffpEnable(FFP_TEXTURE_2D);

		/*
		** Since we want to turn texturing on for floor only, we have to
		** make floor a separate ffp.ffpBegin()/ffp.ffpEnd() sequence. You can't
		** turn texturing on and off between begin and end calls
		*/
		ffp.ffpBegin(FFP_QUADS);
		ffp.ffpNormal3f(0.f, 1.f, 0.f);
		ffp.ffpTexCoord2i(0, 0);
		ffp.ffpVertex3f(-100.f, -100.f, -320.f);
		ffp.ffpTexCoord2i(1, 0);
		ffp.ffpVertex3f(100.f, -100.f, -320.f);
		ffp.ffpTexCoord2i(1, 1);
		ffp.ffpVertex3f(100.f, -100.f, -640.f);
		ffp.ffpTexCoord2i(0, 1);
		ffp.ffpVertex3f(-100.f, -100.f, -640.f);
		ffp.ffpEnd();

		ffp.ffpDisable(FFP_TEXTURE_2D);

		/* walls */

		ffp.ffpBegin(FFP_QUADS);
		/* left wall */
		ffp.ffpNormal3f(1.f, 0.f, 0.f);
		ffp.ffpVertex3f(-100.f, -100.f, -320.f);
		ffp.ffpVertex3f(-100.f, -100.f, -640.f);
		ffp.ffpVertex3f(-100.f, 100.f, -640.f);
		ffp.ffpVertex3f(-100.f, 100.f, -320.f);

		/* right wall */
		ffp.ffpNormal3f(-1.f, 0.f, 0.f);
		ffp.ffpVertex3f(100.f, -100.f, -320.f);
		ffp.ffpVertex3f(100.f, 100.f, -320.f);
		ffp.ffpVertex3f(100.f, 100.f, -640.f);
		ffp.ffpVertex3f(100.f, -100.f, -640.f);

		/* ceiling */
		ffp.ffpNormal3f(0.f, -1.f, 0.f);
		ffp.ffpVertex3f(-100.f, 100.f, -320.f);
		ffp.ffpVertex3f(-100.f, 100.f, -640.f);
		ffp.ffpVertex3f(100.f, 100.f, -640.f);
		ffp.ffpVertex3f(100.f, 100.f, -320.f);

		/* back wall */
		ffp.ffpNormal3f(0.f, 0.f, 1.f);
		ffp.ffpVertex3f(-100.f, -100.f, -640.f);
		ffp.ffpVertex3f(100.f, -100.f, -640.f);
		ffp.ffpVertex3f(100.f, 100.f, -640.f);
		ffp.ffpVertex3f(-100.f, 100.f, -640.f);
		ffp.ffpEnd();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-80.f + dx, -60.f + dy, -420.f + dz);
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, sphere_mat);
		aux.auxSphere(20.f, 20, 20, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_SMOOTH, FFPAUX_FALSE);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-20.f, -80.f, -600.f);
		ffp.ffpRotatef(-90.f, 1.f, 0.f, 0.f);
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, cone_mat);
		aux.auxDisk(0, 20, 20, 1, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_SMOOTH, FFPAUX_FALSE);
		aux.auxCylinder(20, 0, 60, 20, 20, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_SMOOTH, FFPAUX_FALSE);
		ffp.ffpPopMatrix();

		ffp.ffpFlush(FFP_MULTI_THREADED); /* high end machines may need this */
	}

	static final int CONE = 0;
	static final int SPHERE = 1;

}
