package ffp_examples;

/*
 *  polyoff.java
 *  This program demonstrates polygon offset to draw a shaded
 *  polygon and its wireframe counterpart without ugly visual
 *  artifacts ("stitching").
 */
public class polyoff extends DemoBase {

	public static void main(String[] args) {
		new polyoff().init(EUT_RGB);
	}

	static float polyfactor = 1.0f;
	static float polyunits = 1.0f;

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;
	static int xold, yold;

	static float size = 0.5f;
	static int mat_ambient[] = { 156, 156, 156 };
	static int mat_diffuse[] = { 255, 0, 55 };
	static int mat_specular[] = { 255, 255, 255 };
	static int gray[] = { 156, 156, 156 };
	static int black[] = { 0, 0, 0 };

	static float light_ambient[] = { 0, 0, 0 };
	static float light_diffuse[] = { 1, 1, 1 };
	static float light_specular[] = { 1, 1, 1 };
	static float light_position[] = { 1, 1, 1, 0 };

	static int global_ambient[] = { 7, 7, 7 };

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, w / (float) h, 1f, 10.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		aux.auxLookAt(0.0f, 0.0f, 5.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_DEPTH_TEST);

		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpLightModeliv(FFP_LIGHT_MODEL_AMBIENT, global_ambient);
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}

		if (keyPressed(KEY_R)) {
			polyfactor += 0.1f;
			System.out.println("polyfactor is " + polyfactor);
		}
		if (keyPressed(KEY_F)) {
			polyfactor -= 0.1f;
			System.out.println("polyfactor is " + polyfactor);
		}

		if (keyPressed(KEY_T)) {
			polyunits += 0.1f;
			System.out.println("polyunits is " + polyunits);
		}
		if (keyPressed(KEY_G)) {
			polyunits -= 0.1f;
			System.out.println("polyunits is " + polyunits);
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_DEPTH_BUFFER | FFP_VIDEO_BUFFER);

		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);

		ffp.ffpPushMatrix();

		ffp.ffpMaterialiv(FFP_AMBIENT_AND_DIFFUSE, gray);
		ffp.ffpMaterialiv(FFP_SPECULAR, black);
		ffp.ffpMaterialf(FFP_SHININESS, 0.0f);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_POLYGON_OFFSET_FILL);
		ffp.ffpPolygonOffset(polyfactor, polyunits);
		aux.auxSolidSphere(1, 20, 12);
		ffp.ffpDisable(FFP_POLYGON_OFFSET_FILL);

		ffp.ffpDisable(FFP_LIGHTING);
		ffp.ffpDisableLight(0);
		ffp.ffpColor3f(1.0f, 1.0f, 1.0f);
		ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_LINE);
		aux.auxSolidSphere(1, 20, 12);
		ffp.ffpPolygonMode(FFP_FRONT_AND_BACK, FFP_FILL);

		ffp.ffpPopMatrix();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
