package ffp_examples;

/*
 * robot.java
 * This program shows how to composite modeling transformations
 * to draw translated and rotated hierarchical models.
 * Interaction:  pressing the A, W and D, R keys (shoulder and elbow)
 * alters the rotation of the robot arm.
 */
public class robot extends DemoBase {

	public static void main(String[] args) {
		new robot().init(EUT_RGB);
	}

	private static int shoulder = 0, elbow = 0;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 20.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0.0f, 0.0f, -5.0f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
	}

	@Override
	public void dbUpdate() {
		if (keyHeld(KEY_W)) {
			shoulder = (shoulder + 5) % 360;
		}
		if (keyHeld(KEY_A)) {
			shoulder = (shoulder - 5) % 360;
		}
		if (keyHeld(KEY_R)) {
			elbow = (elbow + 5) % 360;
		}
		if (keyHeld(KEY_D)) {
			elbow = (elbow - 5) % 360;
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-1.0f, 0.0f, 0.0f);
		ffp.ffpRotatef(shoulder, 0.0f, 0.0f, 1.0f);
		ffp.ffpTranslatef(1.0f, 0.0f, 0.0f);
		ffp.ffpPushMatrix();
		ffp.ffpScalef(2.0f, 0.4f, 1.0f);
		aux.auxWireCube(.5f);
		ffp.ffpPopMatrix();

		ffp.ffpTranslatef(1.0f, 0.0f, 0.0f);
		ffp.ffpRotatef(elbow, 0.0f, 0.0f, 1.0f);
		ffp.ffpTranslatef(1.0f, 0.0f, 0.0f);
		ffp.ffpPushMatrix();
		ffp.ffpScalef(2.0f, 0.4f, 1.0f);
		aux.auxWireCube(.5f);
		ffp.ffpPopMatrix();

		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
