package ffp_examples;

/*
 * scube.java
 * An advanced program that displays a spinning cube with a dynamic shadow.
 * Press S to toggle stipple blending.
 */
public class scube extends DemoBase {

	static float tick = 0;

	static int GRAY = 0;
	static int RED = 1;
	static int GREEN = 2;
	static int BLUE = 3;
	static int CYAN = 4;
	static int MAGENTA = 5;
	static int YELLOW = 6;
	static int BLACK = 7;

	static float[] square_normal = { 0.0f, 0.0f, 1.0f };
	static float[][] square = new float[4][2];

	static float[][] materialColor = { { 0.8f, 0.8f, 0.8f }, { 0.8f, 0.0f, 0.0f }, { 0.0f, 0.8f, 0.0f }, { 0.0f, 0.0f, 0.8f }, { 0.0f, 0.8f, 0.8f }, { 0.8f, 0.0f, 0.8f }, { 0.8f, 0.8f, 0.0f }, { 0.0f, 0.0f, 0.0f }, };

	static float[] lightPos = { 2.0f, 4.0f, 2.0f, 1.0f };

	static float[] lightAmb = { 0.2f, 0.2f, 0.2f };
	static float[] lightDiff = { 0.8f, 0.8f, 0.8f };
	static float[] lightSpec = { 0.4f, 0.4f, 0.4f };

	static float[] groundPlane = { 0.0f, 1.0f, 0.0f, 1.5f };
	static float[] backPlane = { 0.0f, 0.0f, 1.0f, 0.9f };

	static float[] cubeXform = new float[16];
	static float[] shadowMat = new float[16];

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 0;
	static int xold, yold;
	static short[] pattern = { //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, //n
			0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55 };

	public static void main(String[] args) {
		new scube().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection();
	}

	private static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		ffp.ffpFrustum(-1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 3.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -2);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0x88, 0x88, 0x88);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpLightfv(0, FFP_AMBIENT, lightAmb);
		ffp.ffpLightfv(0, FFP_DIFFUSE, lightDiff);
		ffp.ffpLightfv(0, FFP_SPECULAR, lightSpec);
		ffp.ffpLightfv(0, FFP_POSITION, lightPos);
		ffp.ffpEnable(FFP_NORMALIZE);
		ffp.ffpEnable(FFP_CULL_FACE);
		ffp.ffpCullFace(FFP_BACK);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_FOG);
		ffp.ffpFogColor(0x88, 0x88, 0x88);
		ffp.ffpFogStart(0.95f);
		ffp.ffpPolygonStipple(pattern);
	}

	static void draw_cube(int color) {
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materialColor[color]);
		aux.auxSolidCube(1);
	}

	static void draw_checkerboard(int w, int h, int evenColor, int oddColor) {
		ffp.ffpBegin(FFP_QUADS);
		ffp.ffpNormal3fv(square_normal);
		for (int j = 0; j < h; ++j) {
			for (int i = 0; i < w; ++i) {
				square[0][0] = -1.0f + 2.0f / w * i;
				square[0][1] = -1.0f + 2.0f / h * (j + 1);

				square[1][0] = -1.0f + 2.0f / w * i;
				square[1][1] = -1.0f + 2.0f / h * j;

				square[2][0] = -1.0f + 2.0f / w * (i + 1);
				square[2][1] = -1.0f + 2.0f / h * j;

				square[3][0] = -1.0f + 2.0f / w * (i + 1);
				square[3][1] = -1.0f + 2.0f / h * (j + 1);

				if ((i & 1 ^ j & 1) == 1) {
					ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materialColor[oddColor]);
				} else {
					ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materialColor[evenColor]);
				}

				ffp.ffpVertex2fv(square[0]);
				ffp.ffpVertex2fv(square[1]);
				ffp.ffpVertex2fv(square[2]);
				ffp.ffpVertex2fv(square[3]);
			}
		}
		ffp.ffpEnd();
	}

	static void shadow_matrix(float[] ground, float[] light) {
		float dot = ground[0] * light[0] + ground[1] * light[1] + ground[2] * light[2] + ground[3] * light[3];

		shadowMat[(0 << 2) + 0] = dot - light[0] * ground[0];
		shadowMat[(1 << 2) + 0] = 0.0f - light[0] * ground[1];
		shadowMat[(2 << 2) + 0] = 0.0f - light[0] * ground[2];
		shadowMat[(3 << 2) + 0] = 0.0f - light[0] * ground[3];

		shadowMat[(0 << 2) + 1] = 0.0f - light[1] * ground[0];
		shadowMat[(1 << 2) + 1] = dot - light[1] * ground[1];
		shadowMat[(2 << 2) + 1] = 0.0f - light[1] * ground[2];
		shadowMat[(3 << 2) + 1] = 0.0f - light[1] * ground[3];

		shadowMat[(0 << 2) + 2] = 0.0f - light[2] * ground[0];
		shadowMat[(1 << 2) + 2] = 0.0f - light[2] * ground[1];
		shadowMat[(2 << 2) + 2] = dot - light[2] * ground[2];
		shadowMat[(3 << 2) + 2] = 0.0f - light[2] * ground[3];

		shadowMat[(0 << 2) + 3] = 0.0f - light[3] * ground[0];
		shadowMat[(1 << 2) + 3] = 0.0f - light[3] * ground[1];
		shadowMat[(2 << 2) + 3] = 0.0f - light[3] * ground[2];
		shadowMat[(3 << 2) + 3] = dot - light[3] * ground[3];

		ffp.ffpMultMatrixf(shadowMat);
	}

	static float[] dfloat_to_mat16(float[][] a) {
		float[] m = new float[16];

		m[(0 << 2) + 0] = a[0][0];
		m[(1 << 2) + 0] = a[1][0];
		m[(2 << 2) + 0] = a[2][0];
		m[(3 << 2) + 0] = a[3][0];

		m[(0 << 2) + 1] = a[0][1];
		m[(1 << 2) + 1] = a[1][1];
		m[(2 << 2) + 1] = a[2][1];
		m[(3 << 2) + 1] = a[3][1];

		m[(0 << 2) + 2] = a[0][2];
		m[(1 << 2) + 2] = a[1][2];
		m[(2 << 2) + 2] = a[2][2];
		m[(3 << 2) + 2] = a[3][2];

		m[(0 << 2) + 3] = a[0][3];
		m[(1 << 2) + 3] = a[1][3];
		m[(2 << 2) + 3] = a[2][3];
		m[(3 << 2) + 3] = a[3][3];

		return m;
	}

	static boolean stipple_blend = false;

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_S)) {
			stipple_blend = !stipple_blend;
			ffp.ffpVideoPropertyi(FFP_VIDEO_BLEND_STIPPLED_REGIONS, stipple_blend ? FFP_TRUE : FFP_FALSE);
		}
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < -0.85f) {
				translate_z = -0.85f;
			}
			if (translate_z > 1.25f) {
				translate_z = 1.25f;
			}
		}
		xold = x;
		yold = y;
		tick += 0.1f;
		tick %= 120;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.0f, -1.5f, 0.0f);
		ffp.ffpRotatef(-90.0f, 1, 0, 0);
		ffp.ffpScalef(2.0f, 2.0f, 2.0f);

		draw_checkerboard(6, 6, BLUE, YELLOW); /* draw ground */
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.0f, 0.0f, -0.9f);
		ffp.ffpScalef(2.0f, 2.0f, 2.0f);

		draw_checkerboard(6, 6, BLUE, YELLOW); /* draw back */
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.0f, 0.2f, 0.0f);
		ffp.ffpScalef(0.3f, 0.3f, 0.3f);
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);
		ffp.ffpScalef(1.0f, 2.0f, 1.0f);
		ffp.ffpGetFloatv(FFP_MODELVIEW_MATRIX, cubeXform);

		draw_cube(RED); /* draw cube */
		ffp.ffpPopMatrix();

		ffp.ffpDepthWritemask(FFP_FALSE);
		ffp.ffpEnable(FFP_POLYGON_STIPPLE);
		ffp.ffpPushMatrix();
		shadow_matrix(groundPlane, lightPos);
		ffp.ffpTranslatef(0.0f, 0.0f, 2.0f);
		ffp.ffpMultMatrixf(cubeXform);
		ffp.ffpEnable(FFP_POLYGON_OFFSET_FILL);
		ffp.ffpPolygonOffset(-0.1f, -0.1f);
		draw_cube(BLACK); /* draw ground shadow */
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		shadow_matrix(backPlane, lightPos);
		ffp.ffpTranslatef(0.0f, 0.0f, 2f);
		ffp.ffpMultMatrixf(cubeXform);

		draw_cube(BLACK); /* draw back shadow */
		ffp.ffpPopMatrix();
		ffp.ffpDisable(FFP_POLYGON_OFFSET_FILL);
		ffp.ffpDepthWritemask(FFP_TRUE);
		ffp.ffpDisable(FFP_POLYGON_STIPPLE);
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}