package ffp_examples;

/*
 *  shadowmap.java
 *  This program demonstrates painfully slow post-render
 *  shadow mapping.
 */
public class shadowmap extends DemoBase {

	public static void main(String[] args) {
		new shadowmap().init(EUT_RGB);
	}

	static int GRAY = 0;
	static int RED = 1;
	static int GREEN = 2;
	static int BLUE = 3;
	static int CYAN = 4;
	static int MAGENTA = 5;
	static int YELLOW = 6;
	static int BLACK = 7;

	static float light_position[] = { 6.171117f, 11f, 16.607384f, 1.0f };

	static float rotate_x = 30;
	static int xold, yold;

	static float polyfactor = 6.2f;
	static float polyunits = 0.0f;

	static float[] light_depth;
	static float[] view_depth;

	static int[] viewport = new int[4];
	static float[] cmvmatrix = new float[16], cprojmatrix = new float[16];
	static float[] lmvmatrix = new float[16], lprojmatrix = new float[16];

	static float[] win = new float[3];
	static float[] wp = new float[3];

	static float[][] square = new float[4][3];

	static float[][] materialColor = { { 0.5f, 0.5f, 0.5f }, { 0.5f, 0.1f, 0.1f }, { 0.1f, 0.5f, 0.1f }, { 0.1f, 0.1f, 0.5f }, { 0.1f, 0.5f, 0.5f }, { 0.5f, 0.1f, 0.5f }, { 0.5f, 0.5f, 0.1f }, { 0.1f, 0.1f, 0.1f } };

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 300.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		light_depth = new float[width * height];
		view_depth = new float[width * height];
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_CULL_FACE);
		ffp.ffpCullFace(FFP_BACK);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);

		ffp.ffpLightfv(0, FFP_POSITION, light_position);
	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
		}

		if (keyPressed(KEY_R)) {
			polyfactor += 0.1f;
			System.out.println("polyfactor is " + polyfactor);
		}
		if (keyPressed(KEY_F)) {
			polyfactor -= 0.1f;
			System.out.println("polyfactor is " + polyfactor);
		}

		if (keyPressed(KEY_T)) {
			polyunits += 0.1f;
			System.out.println("polyunits is " + polyunits);
		}
		if (keyPressed(KEY_G)) {
			polyunits -= 0.1f;
			System.out.println("polyunits is " + polyunits);
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpGetIntegerv(FFP_VIEWPORT, viewport);

		{
			ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
			ffp.ffpPushMatrix();
			{
				ffp.ffpRotatef(36, 1, 0, 0);
				ffp.ffpRotatef(-24, 0, 1, 0);
				ffp.ffpTranslatef(-light_position[0], -light_position[1], -light_position[2]);
				ffp.ffpGetFloatv(FFP_MODELVIEW_MATRIX, lmvmatrix);
				ffp.ffpGetFloatv(FFP_PROJECTION_MATRIX, lprojmatrix);

				drawScene(true);
			}
			ffp.ffpPopMatrix();
			ffp.ffpFlush(FFP_MULTI_THREADED);
		}

		ffp.ffpGetFloatv(FFP_DEPTH_BUFFER, light_depth);

		{
			ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
			ffp.ffpPushMatrix();
			{
				aux.auxLookAt(10, 5, 10, 0, 0, 0, 0, 1, 0);
				ffp.ffpGetFloatv(FFP_MODELVIEW_MATRIX, cmvmatrix);
				ffp.ffpGetFloatv(FFP_PROJECTION_MATRIX, cprojmatrix);

				drawScene(false);
			}
			ffp.ffpPopMatrix();
			ffp.ffpFlush(FFP_MULTI_THREADED);
		}

		ffp.ffpGetFloatv(FFP_DEPTH_BUFFER, view_depth);

		int[] vmem = video;
		int vwd = dbWidth;
		int vht = dbHeight;
		int sz = vwd * vht;
		int x = 0, y = 0;
		int width_m2 = vwd - 2;
		for (int i = 0; i < sz; i++) {
			if ((vmem[i] & 0x00ffffff) != 0) {
				aux.auxUnProject(x, toFFPY(viewport, y), view_depth[i], cmvmatrix, cprojmatrix, viewport, win);
				aux.auxProject(win[0], win[1], win[2], lmvmatrix, lprojmatrix, viewport, wp);
				int mapIndex = (int) (wp[0] + toFFPY(viewport, wp[1]) * vwd);
				if (mapIndex >= 0 && mapIndex < light_depth.length) {
					if (light_depth[mapIndex] - wp[2] < -0.0001) {
						vmem[i] = darken(vmem[i]);
					}
				}
			}
			if (x > width_m2) {
				y++;
				x = 0;
			} else {
				x++;
			}
		}
	}

	private void drawScene(boolean lightpov) {
		if (!lightpov) {
			ffp.ffpPushMatrix();
			ffp.ffpLightfv(0, FFP_POSITION, light_position);
			ffp.ffpEnable(FFP_LIGHTING);
			ffp.ffpDisable(FFP_POLYGON_OFFSET_FILL);
			ffp.ffpPopMatrix();
		} else {
			ffp.ffpDisable(FFP_LIGHTING);
			ffp.ffpEnable(FFP_POLYGON_OFFSET_FILL);
			ffp.ffpPolygonOffset(polyfactor, polyunits);
		}
		ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materialColor[GRAY]);
		ffp.ffpPushMatrix();
		{
			ffp.ffpRotatef(rotate_x, 0, 1.0f, 0);
			aux.auxSolidTorus(0.5f, 1, 32, 32);
			ffp.ffpRotatef(rotate_x, 0, 1.0f, 0);
			aux.auxSolidTorus(0.5f, 5, 32, 32);
		}
		ffp.ffpPopMatrix();
		ffp.ffpPushMatrix();
		{
			ffp.ffpTranslatef(0, 0, 5.0f);
			aux.auxSolidCube(1);
		}
		ffp.ffpPopMatrix();
		{
			draw_checkerboard(10, 10, 10, YELLOW, BLUE);
		}
	}

	static void draw_checkerboard(int w, int h, float scale, int evenColor, int oddColor) {
		ffp.ffpBegin(FFP_QUADS);
		ffp.ffpNormal3f(0, 1, 0);
		for (int j = 0; j < h; ++j) {
			for (int i = 0; i < w; ++i) {
				square[0][0] = scale * (-1.0f + 2.0f / w * i);
				square[0][1] = -2f;
				square[0][2] = scale * (-1.0f + 2.0f / h * (j + 1));

				square[1][0] = scale * (-1.0f + 2.0f / w * i);
				square[1][1] = -2f;
				square[1][2] = scale * (-1.0f + 2.0f / h * j);

				square[2][0] = scale * (-1.0f + 2.0f / w * (i + 1));
				square[2][1] = -2f;
				square[2][2] = scale * (-1.0f + 2.0f / h * j);

				square[3][0] = scale * (-1.0f + 2.0f / w * (i + 1));
				square[3][1] = -2f;
				square[3][2] = scale * (-1.0f + 2.0f / h * (j + 1));

				if ((i & 1 ^ j & 1) == 1) {
					ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materialColor[oddColor]);
				} else {
					ffp.ffpMaterialfv(FFP_AMBIENT_AND_DIFFUSE, materialColor[evenColor]);
				}

				ffp.ffpVertex3fv(square[3]);
				ffp.ffpVertex3fv(square[2]);
				ffp.ffpVertex3fv(square[1]);
				ffp.ffpVertex3fv(square[0]);
			}
		}
		ffp.ffpEnd();
	}

	private int darken(int color) {
		return color >> 1 & 0x7f7f7f;
	}

	private int toFFPY(int[] viewport, float y) {
		return (int) (viewport[3] - y - 1);
	}

}
