package ffp_examples;

/*
 *  smoothtorus.java
 */
public class smoothtorus extends DemoBase {

	public static void main(String[] args) {
		new smoothtorus().init(EUT_RGB);
	}

	static float light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	/* light_position is NOT default value */
	static float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };

	static float rotate_x = 30;
	static float rotate_y = 15;
	static float translate_z = 4;
	static int xold, yold;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / (float) h, 1.0f, 300.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpSmoothModel(FFP_PHONG);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);

		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);
		ffp.ffpMaterialfv(FFP_DIFFUSE, light_diffuse);
	}

	private boolean spec = false;

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		if (keyPressed(KEY_S)) {
			spec = !spec;
			if (spec) {
				ffp.ffpMaterialfv(FFP_SPECULAR, light_specular);
				ffp.ffpMaterialf(FFP_SHININESS, 100);
			} else {
				ffp.ffpMaterialfv(FFP_SPECULAR, new float[] { 0, 0, 0 });
				ffp.ffpMaterialf(FFP_SHININESS, 0);
			}
		}
		xold = x;
		yold = y;
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		{
			ffp.ffpLoadIdentity();
			ffp.ffpTranslatef(0, 0, -translate_z);
			ffp.ffpRotatef(rotate_y, 1, 0, 0);
			ffp.ffpRotatef(rotate_x, 0, 1, 0);
			aux.auxSolidTorus(0.275f, 0.85f, 64, 64);
		}
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
