package ffp_examples;

/*
 *  spheres.java
 *  This program demonstrates lots of material properties.
 *  A single light source illuminates the objects.
 */
public class spheres extends DemoBase {

	public static void main(String[] args) {
		new spheres().init(EUT_RGB | EUT_NOT_RESIZABLE);
	}

	static float ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float position[] = { 0.0f, 3.0f, 3.0f, 0.0f };

	static float lmodel_ambient[] = { 0.2f, 0.2f, 0.2f, 1.0f };

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		float v = 20f;
		if (w <= h) {
			ffp.ffpOrtho(0.0f, v, 0.0f, v * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(0.0f, v * w / h, 0.0f, v, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_NORMALIZE);
		ffp.ffpEnable(FFP_DEPTH_TEST);

		ffp.ffpLightfv(0, FFP_AMBIENT, ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, diffuse);
		ffp.ffpLightfv(0, FFP_POSITION, position);
		ffp.ffpLightModelfv(FFP_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_FALSE);
	}

	@Override
	public void dbUpdate() {

	}

	/*
	 *  1st row:  emerald, brass, black, black
	 *  2nd row:  jade, bronze, cyan, cyan
	 *  3rd row:  obsidian, chrome, green, green
	 *  4th row:  pearl, copper, red, red
	 *  5th row:  ruby, gold, white, white
	 *  6th row:  turquoise, silver, yellow plastic, yellow rubber
	 */
	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(1.5f, 0, 0);
		render_object(2.0f, 17.0f, 0.0215f, 0.1745f, 0.0215f, 0.07568f, 0.61424f, 0.07568f, 0.633f, 0.727811f, 0.633f, 0.6f);
		render_object(2.0f, 14.0f, 0.135f, 0.2225f, 0.1575f, 0.54f, 0.89f, 0.63f, 0.316228f, 0.316228f, 0.316228f, 0.1f);
		render_object(2.0f, 11.0f, 0.05375f, 0.05f, 0.06625f, 0.18275f, 0.17f, 0.22525f, 0.332741f, 0.328634f, 0.346435f, 0.3f);
		render_object(2.0f, 8.0f, 0.25f, 0.20725f, 0.20725f, 1f, 0.829f, 0.829f, 0.296648f, 0.296648f, 0.296648f, 0.088f);
		render_object(2.0f, 5.0f, 0.1745f, 0.01175f, 0.01175f, 0.61424f, 0.04136f, 0.04136f, 0.727811f, 0.626959f, 0.626959f, 0.6f);
		render_object(2.0f, 2.0f, 0.1f, 0.18725f, 0.1745f, 0.396f, 0.74151f, 0.69102f, 0.297254f, 0.30829f, 0.306678f, 0.1f);
		render_object(6.0f, 17.0f, 0.329412f, 0.223529f, 0.027451f, 0.780392f, 0.568627f, 0.113725f, 0.992157f, 0.941176f, 0.807843f, 0.21794872f);
		render_object(6.0f, 14.0f, 0.2125f, 0.1275f, 0.054f, 0.714f, 0.4284f, 0.18144f, 0.393548f, 0.271906f, 0.166721f, 0.2f);
		render_object(6.0f, 11.0f, 0.25f, 0.25f, 0.25f, 0.4f, 0.4f, 0.4f, 0.774597f, 0.774597f, 0.774597f, 0.6f);
		render_object(6.0f, 8.0f, 0.19125f, 0.0735f, 0.0225f, 0.7038f, 0.27048f, 0.0828f, 0.256777f, 0.137622f, 0.086014f, 0.1f);
		render_object(6.0f, 5.0f, 0.24725f, 0.1995f, 0.0745f, 0.75164f, 0.60648f, 0.22648f, 0.628281f, 0.555802f, 0.366065f, 0.4f);
		render_object(6.0f, 2.0f, 0.19225f, 0.19225f, 0.19225f, 0.50754f, 0.50754f, 0.50754f, 0.508273f, 0.508273f, 0.508273f, 0.4f);
		render_object(10.0f, 17.0f, 0.0f, 0.0f, 0.0f, 0.01f, 0.01f, 0.01f, 0.50f, 0.50f, 0.50f, .25f);
		render_object(10.0f, 14.0f, 0.0f, 0.1f, 0.06f, 0.0f, 0.50980392f, 0.50980392f, 0.50196078f, 0.50196078f, 0.50196078f, .25f);
		render_object(10.0f, 11.0f, 0.0f, 0.0f, 0.0f, 0.1f, 0.35f, 0.1f, 0.45f, 0.55f, 0.45f, .25f);
		render_object(10.0f, 8.0f, 0.0f, 0.0f, 0.0f, 0.5f, 0.0f, 0.0f, 0.7f, 0.6f, 0.6f, .25f);
		render_object(10.0f, 5.0f, 0.0f, 0.0f, 0.0f, 0.55f, 0.55f, 0.55f, 0.70f, 0.70f, 0.70f, .25f);
		render_object(10.0f, 2.0f, 0.0f, 0.0f, 0.0f, 0.5f, 0.5f, 0.0f, 0.60f, 0.60f, 0.50f, .25f);
		render_object(14.0f, 17.0f, 0.02f, 0.02f, 0.02f, 0.01f, 0.01f, 0.01f, 0.4f, 0.4f, 0.4f, .078125f);
		render_object(14.0f, 14.0f, 0.0f, 0.05f, 0.05f, 0.4f, 0.5f, 0.5f, 0.04f, 0.7f, 0.7f, .078125f);
		render_object(14.0f, 11.0f, 0.0f, 0.05f, 0.0f, 0.4f, 0.5f, 0.4f, 0.04f, 0.7f, 0.04f, .078125f);
		render_object(14.0f, 8.0f, 0.05f, 0.0f, 0.0f, 0.5f, 0.4f, 0.4f, 0.7f, 0.04f, 0.04f, .078125f);
		render_object(14.0f, 5.0f, 0.05f, 0.05f, 0.05f, 0.5f, 0.5f, 0.5f, 0.7f, 0.7f, 0.7f, .078125f);
		render_object(14.0f, 2.0f, 0.05f, 0.05f, 0.0f, 0.5f, 0.5f, 0.4f, 0.7f, 0.7f, 0.04f, .078125f);
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	/*
	 *  Move object into position.  Use 3rd through 12th parameters
	 *  to specify the material property.  Draw a sphere.
	 */

	static float[] mat = new float[3];

	private static void render_object(float x, float y, float ambr, float ambg, float ambb, float difr, float difg, float difb, float specr, float specg, float specb, float shine) {
		ffp.ffpPushMatrix();
		ffp.ffpScalef(1.05f, 1.05f, 1.05f);
		ffp.ffpTranslatef(x, y, 0.0f);
		mat[0] = ambr;
		mat[1] = ambg;
		mat[2] = ambb;
		ffp.ffpMaterialfv(FFP_AMBIENT, mat);
		mat[0] = difr;
		mat[1] = difg;
		mat[2] = difb;
		ffp.ffpMaterialfv(FFP_DIFFUSE, mat);
		mat[0] = specr;
		mat[1] = specg;
		mat[2] = specb;
		ffp.ffpMaterialfv(FFP_SPECULAR, mat);
		ffp.ffpMaterialf(FFP_SHININESS, shine * 128.0f);
		aux.auxSolidSphere(1, 16, 16);
		ffp.ffpPopMatrix();
	}

}
