package ffp_examples;

/*
 * star.java
 * 3D starfield
 */
public class star extends DemoBase {

	static final float PI = 3.141592657f;

	static final int MAXSTARS = 400;
	static final int MAXPOS = 10000;
	static final int MAXWARP = 500;
	static final int MAXANGLES = 6000;

	static final int NORMAL = 0;
	static final int WEIRD = 1;

	static final int STREAK = 0;
	static final int CIRCLE = 1;

	static class starRec {
		int type;
		float[] x = new float[2];
		float[] y = new float[2];
		float[] z = new float[2];
		float offsetX, offsetY, offsetR, rotation;
	}

	static int flag = NORMAL;
	static int starCount = MAXSTARS / 2;
	static float speed = 1.0f;
	static int nitro = 0;
	static starRec[] stars = new starRec[MAXSTARS];
	static double t0 = -1.;
	static float[] sinTable = new float[MAXANGLES];
	static float[] tx0 = new float[1];
	static float[] tx1 = new float[1];
	static float[] ty0 = new float[1];
	static float[] ty1 = new float[1];
	static float time = 0;

	public static void main(String[] args) {
		new star().init(EUT_RGB);
	}

	public void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection();
	}

	private void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxOrtho2D(-0.5f, 500.5f, -0.5f, 500.5f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		float angle;
		int n;

		for (n = 0; n < MAXSTARS; n++) {
			stars[n] = new starRec();
			new_star(n, 100);
		}

		angle = 0.0f;
		for (n = 0; n < MAXANGLES; n++) {
			sinTable[n] = DemoBase.sin(angle);
			angle += PI / (MAXANGLES / 2.0);
		}
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_T)) {
			nitro = nitro == 0 ? 1 : 0;
		}
		if (keyPressed(KEY_SPACE)) {
			flag = (flag == NORMAL) ? WEIRD : NORMAL;
		}
		time++;
	}

	@Override
	public void dbRender() {
		move_stars();
		ffp.ffpClear(FFP_VIDEO_BUFFER);

		for (int n = 0; n < starCount; n++) {
			if (stars[n].z[0] > speed || (stars[n].z[0] > 0.0 && speed < MAXWARP)) {
				if (star_point(n) == FFP_FALSE) {
					new_star(n, MAXPOS);
				}
			} else {
				new_star(n, MAXPOS);
			}
		}

		if (nitro > 0) {
			speed = nitro / 10f + 1.0f;
			if (speed > MAXWARP) {
				speed = MAXWARP;
			}
			if (++nitro > MAXWARP * 10) {
				nitro = -nitro;
			}
		} else if (nitro < 0) {
			nitro++;
			speed = -nitro / 10f + 1.0f;
			if (speed > MAXWARP) {
				speed = MAXWARP;
			}
		}
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

	float sin(float angle) {
		return sinTable[(int) angle];
	}

	float cos(float angle) {
		return (sinTable[((int) angle + (MAXANGLES / 4)) % MAXANGLES]);
	}

	void new_star(int n, int d) {
		if (random() < 0.25) {
			stars[n].type = CIRCLE;
		} else {
			stars[n].type = STREAK;
		}
		stars[n].x[0] = random() * MAXPOS - MAXPOS / 2;
		stars[n].y[0] = random() * MAXPOS - MAXPOS / 2;
		stars[n].z[0] = random() * MAXPOS + d;
		if (flag == WEIRD) {
			stars[n].offsetX = random() * 100 - 100 / 2;
			stars[n].offsetY = random() * 100 - 100 / 2;
			stars[n].offsetR = random() * 25 - 25 / 2;
		} else {
			stars[n].offsetX = 0.0f;
			stars[n].offsetY = 0.0f;
			stars[n].offsetR = 0.0f;
		}
	}

	void rotate_point(float[] x, float[] y, float rotation) {
		float tmpX = x[0] * cos(rotation) - y[0] * sin(rotation);
		float tmpY = y[0] * cos(rotation) + x[0] * sin(rotation);
		x[0] = tmpX;
		y[0] = tmpY;
	}

	void move_stars() {
		float offset;
		int n;

		double t, dt;
		t = time / 1000.0;
		if (t0 < 0.)
			t0 = t;
		dt = 85. * (t - t0);
		t0 = t;

		offset = speed * 60.0f;

		for (n = 0; n < starCount; n++) {
			stars[n].x[1] = stars[n].x[0];
			stars[n].y[1] = stars[n].y[0];
			stars[n].z[1] = stars[n].z[0];
			stars[n].x[0] += stars[n].offsetX * dt;
			stars[n].y[0] += stars[n].offsetY * dt;
			stars[n].z[0] -= offset * dt;
			stars[n].rotation += stars[n].offsetR * dt;
			if (stars[n].rotation > MAXANGLES) {
				stars[n].rotation = 0.0f;
			} else if (stars[n].rotation < 0.0) {
				stars[n].rotation += 360.0;
			}
		}
	}

	int star_point(int n) {
		tx0[0] = stars[n].x[0] * dbWidth / stars[n].z[0];
		ty0[0] = stars[n].y[0] * dbHeight / stars[n].z[0];
		rotate_point(tx0, ty0, stars[n].rotation);
		tx0[0] += dbWidth / 2.0;
		ty0[0] += dbHeight / 2.0;

		if (tx0[0] >= 0.0 && tx0[0] < dbWidth && ty0[0] >= 0.0 && ty0[0] < dbHeight) {
			if (stars[n].type == STREAK) {
				tx1[0] = stars[n].x[1] * dbWidth / stars[n].z[1];
				ty1[0] = stars[n].y[1] * dbHeight / stars[n].z[1];
				rotate_point(tx1, ty1, stars[n].rotation);
				tx1[0] += dbWidth / 2.0;
				ty1[0] += dbHeight / 2.0;

				//glLineWidth(MAXPOS / 100.0 / stars[n].z[0] + 1.0);
				ffp.ffpColor3f(1.0f, (MAXWARP - speed) / MAXWARP, (MAXWARP - speed) / MAXWARP);
				if (abs(tx0[0] - tx1[0]) < 1.0 && abs(ty0[0] - ty1[0]) < 1.0) {
					ffp.ffpBegin(FFP_POINTS);
					ffp.ffpVertex2f(tx0[0], ty0[0]);
					ffp.ffpEnd();
				} else {
					ffp.ffpBegin(FFP_LINES);
					ffp.ffpVertex2f(tx0[0], ty0[0]);
					ffp.ffpVertex2f(tx1[0], ty1[0]);
					ffp.ffpEnd();
				}
			} else {
				float width = MAXPOS / 10.0f / stars[n].z[0] + 1.0f;
				ffp.ffpColor3f(1.0f, 1.0f, 0.0f);
				float x0 = tx0[0] + width * cos(0);
				float y0 = ty0[0] + width * sin(0);
				ffp.ffpBegin(FFP_TRIANGLES);
				for (int i = 2; i < 8; i++) {
					ffp.ffpVertex2f(x0, y0);
					float x = tx0[0] + width * cos((i - 1) * MAXANGLES / 8.0f);
					float y = ty0[0] + width * sin((i - 1) * MAXANGLES / 8.0f);
					ffp.ffpVertex2f(x, y);
					float x1 = tx0[0] + width * cos(i * MAXANGLES / 8.0f);
					float y1 = ty0[0] + width * sin(i * MAXANGLES / 8.0f);
					ffp.ffpVertex2f(x1, y1);
				}
				ffp.ffpEnd();
			}
			return FFP_TRUE;
		}
		return FFP_FALSE;
	}

}
