package ffp_examples;

/*
 *  texsub.java
 *  This program texture maps a checkerboard image onto
 *  two rectangles.  This program clamps the texture, if
 *  the texture coordinates fall outside 0.0 and 1.0.
 *  If the s key is pressed, a texture subimage is used to
 *  alter the original texture.  If the r key is pressed, 
 *  the original texture is restored.
 */
public class texsub extends DemoBase {

	public static void main(String[] args) {
		new texsub().init(EUT_RGB);
	}

	static int tex_width = 64;
	static int tex_height = 64;
	static int[] tex = new int[tex_width * tex_height];

	static int sub_width = 16;
	static int sub_height = 16;
	static int[] subtex = new int[sub_width * sub_height];

	static void gen_textures() {
		for (int i = 0; i < tex_width; i++) {
			for (int j = 0; j < tex_height; j++) {
				if ((((i & 0x8) == 0) ^ ((j & 0x8)) == 0)) {
					tex[i + j * tex_width] = 0xffffffff;
				} else {
					tex[i + j * tex_width] = 0;
				}
			}
		}

		for (int i = 0; i < sub_width; i++) {
			for (int j = 0; j < sub_height; j++) {
				if ((((i & 0x4) == 0) ^ ((j & 0x4)) == 0)) {
					subtex[i + j * sub_width] = 0xffff0000;
				} else {
					subtex[i + j * sub_width] = 0;
				}
			}
		}
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(60.0f, (float) w / h, 1.0f, 30.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -3.6f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_FLAT);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_TEXTURE_2D);
		ffp.ffpEnable(FFP_ALPHA_TEST);
		gen_textures();
		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_FILTER, FFP_NEAREST);
		ffp.ffpTexParameteri(FFP_TEXTURE_2D, FFP_TEXTURE_ENV_MODE, FFP_DECAL);
		ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed('s')) {
			ffp.ffpTexSubImage2D(FFP_TEXTURE_2D, 12, 44, sub_width, sub_height, subtex);
		}
		if (keyPressed('r')) {
			ffp.ffpTexImage2D(FFP_TEXTURE_2D, FFP_RGB, tex, tex_width, tex_height);
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		ffp.ffpBindTexture(FFP_TEXTURE_2D, 0);
		ffp.ffpBegin(FFP_QUADS);
		ffp.ffpTexCoord2f(0.0f, 0.0f);
		ffp.ffpVertex3f(-2.0f, -1.0f, 0.0f);
		ffp.ffpTexCoord2f(0.0f, 1.0f);
		ffp.ffpVertex3f(-2.0f, 1.0f, 0.0f);
		ffp.ffpTexCoord2f(1.0f, 1.0f);
		ffp.ffpVertex3f(0.0f, 1.0f, 0.0f);
		ffp.ffpTexCoord2f(1.0f, 0.0f);
		ffp.ffpVertex3f(0.0f, -1.0f, 0.0f);

		ffp.ffpTexCoord2f(0.0f, 0.0f);
		ffp.ffpVertex3f(1.0f, -1.0f, 0.0f);
		ffp.ffpTexCoord2f(0.0f, 1.0f);
		ffp.ffpVertex3f(1.0f, 1.0f, 0.0f);
		ffp.ffpTexCoord2f(1.0f, 1.0f);
		ffp.ffpVertex3f(2.41421f, 1.0f, -1.41421f);
		ffp.ffpTexCoord2f(1.0f, 0.0f);
		ffp.ffpVertex3f(2.41421f, -1.0f, -1.41421f);
		ffp.ffpEnd();

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
