package ffp_examples;

/*
 *  vidfilter.java
 *  This program demonstrates the three image
 *  filters available from FFP. Press E to turn
 *  on the edge detection filter, press S to
 *  turn on the sharp filter, press R to turn
 *  on the raised filter, and press N to turn
 *  off filtering. Flat/Smooth shading is
 *  toggleable by pressing F.
 */
public class vidfilter extends DemoBase {

	static float light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	static float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	static float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	/* light_position is NOT default value */
	static float light_position[] = { 1.0f, 1.0f, 1.0f, 0.0f };

	public static void main(String[] args) {
		new vidfilter().init(EUT_RGB);
	}

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		float v = 2.5f;
		if (w <= h) {
			ffp.ffpOrtho(-v, v, -v * h / w, v * h / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-v * w / h, v * w / h, -v, v, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);

		ffp.ffpLightfv(0, FFP_AMBIENT, light_ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, light_diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, light_specular);
		ffp.ffpLightfv(0, FFP_POSITION, light_position);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_DEPTH_TEST);
	}

	@Override
	public void dbUpdate() {
		if (keyPressed(KEY_E)) {
			ffp.ffpVideoPropertyi(FFP_VIDEO_FILTER, FFP_VIDEO_FILTER_EDGE_DETECTION);
		}
		if (keyPressed(KEY_S)) {
			ffp.ffpVideoPropertyi(FFP_VIDEO_FILTER, FFP_VIDEO_FILTER_SHARP);
		}
		if (keyPressed(KEY_R)) {
			ffp.ffpVideoPropertyi(FFP_VIDEO_FILTER, FFP_VIDEO_FILTER_RAISED);
		}
		if (keyPressed(KEY_N)) {
			ffp.ffpVideoPropertyi(FFP_VIDEO_FILTER, FFP_VIDEO_FILTER_NONE);
		}
		if (keyPressed(KEY_F)) {
			boolean state = ffp.ffpGetInteger(FFP_SHADE_MODEL) == FFP_SMOOTH;
			if (state) {
				ffp.ffpShadeModel(FFP_FLAT);
			} else {
				ffp.ffpShadeModel(FFP_SMOOTH);
			}
		}
	}

	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		ffp.ffpPushMatrix();
		ffp.ffpRotatef(20.0f, 1.0f, 0.0f, 0.0f);

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.75f, 0.5f, 0.0f);
		ffp.ffpRotatef(90.0f, 1.0f, 0.0f, 0.0f);
		aux.auxSolidTorus(0.275f, 0.85f, 15, 15);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-0.75f, -0.5f, 0.0f);
		ffp.ffpRotatef(270.0f, 1.0f, 0.0f, 0.0f);
		aux.auxCone(1.0f, 2.0f, 15, 15, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_SMOOTH, FFPAUX_FALSE);
		ffp.ffpPopMatrix();

		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(0.75f, 0.0f, -1.0f);
		aux.auxSphere(1.0f, 15, 15, FFPAUX_FILL, FFPAUX_OUTSIDE, FFPAUX_SMOOTH, FFPAUX_FALSE);
		ffp.ffpPopMatrix();

		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
