package ffp_examples;

/*
 *  wave.java
 */
public class wave extends DemoBase {

	public static void main(String[] args) {
		new wave().init(EUT_RGB);
	}

	static _face get_face(int frame, int x, int y) {
		return mesh.faces[frame * mesh.numFacets + x + y * (mesh.width)];
	}

	static _vertex get_vert(int frame, int x, int y) {
		return mesh.verts[frame * mesh.numCoords + x + y * (mesh.width + 1)];
	}

	static class _face {
		float[] color = new float[3];
		float[] norm = new float[3];
	}

	static class _vertex {
		float[] pos = new float[3];
		float[] norm = new float[3];
	}

	static class _mesh {
		int width, height;
		int numFacets;
		int numCoords;
		int frames;
		_vertex[] verts;
		_face[] faces;
	}

	static float[] check_color_0 = { 0.6f, 0.1f, 0.1f };
	static float[] check_color_1 = { 0.1f, 0.6f, 0.1f };

	static int mesh_width = 20, mesh_height = 20;

	static int checker_size = 2;
	static float amp = 0.3f;

	static int frames = 100, cur_frame = 0;

	static _mesh mesh;

	static float ambient[] = { 0.1f, 0.1f, 0.1f };
	static float diffuse[] = { 1.0f, 1.0f, 1.0f };
	static float position[] = { 1, -1, -1, 0 };
	static float m_spec[] = { 1.0f, 1.0f, 1.0f };
	static float m_shine[] = { 60.0f };

	static float rotate_x = 6;
	static float rotate_y = -50;
	static float translate_z = 2;
	static int xold, yold;

	static final float TWO_PI = 6.2831855f;

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection();
	}

	static void create_projection() {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		aux.auxPerspective(45, 1.0f, 0.5f, 10.0f);
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0.0f, 0.0f, -2.0f);
		ffp.ffpRotatef(-35.0f, 1.0f, 0.0f, 0.0f);
		ffp.ffpRotatef(35.0f, 0.0f, 0.0f, 1.0f);
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 0, 0);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpSmoothModel(FFP_PHONG);

		ffp.ffpEnable(FFP_DEPTH_TEST);
		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
		ffp.ffpEnable(FFP_COLOR_MATERIAL);
		ffp.ffpLightfv(0, FFP_AMBIENT, ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, diffuse);
		ffp.ffpLightfv(0, FFP_SPECULAR, diffuse);
		ffp.ffpLightfv(0, FFP_POSITION, position);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_TWO_SIDE, FFP_TRUE);

		ffp.ffpMaterialfv(FFP_SHININESS, m_shine);
		ffp.ffpMaterialfv(FFP_SPECULAR, m_spec);
		ffp.ffpColorMaterial(FFP_DIFFUSE);

		_vertex vert;
		_face face;
		float dp1[], dp2[];
		float pt1[], pt2[], pt3[];
		float angle, d, x, y;
		int numFacets, numCoords, frameNum, i, j;

		dp1 = new float[3];
		dp2 = new float[3];

		mesh = new _mesh();
		mesh.width = mesh_width;
		mesh.height = mesh_height;
		mesh.frames = frames;

		numFacets = mesh_width * mesh_height;
		numCoords = (mesh_width + 1) * (mesh_height + 1);

		mesh.numCoords = numCoords;
		mesh.numFacets = numFacets;

		mesh.verts = new _vertex[frames * numCoords];
		for (i = 0; i < frames * numCoords; i++)
			mesh.verts[i] = new _vertex();

		mesh.faces = new _face[frames * numFacets];
		for (i = 0; i < frames * numFacets; i++)
			mesh.faces[i] = new _face();

		for (frameNum = 0; frameNum < frames; frameNum++) {
			for (i = 0; i <= mesh_width; i++) {
				x = i / (float) mesh_width;
				for (j = 0; j <= mesh_height; j++) {
					y = j / (float) mesh_height;

					d = sqrt(x * x + y * y);
					if (d == 0.0) {
						d = 0.0001f;
					}
					angle = TWO_PI * d + (TWO_PI / frames * frameNum);

					vert = get_vert(frameNum, i, j);

					vert.pos[0] = x - 0.5f;
					vert.pos[1] = y - 0.5f;
					vert.pos[2] = (amp - amp * d) * cos(angle);

					vert.norm[0] = -(amp / d) * x * ((1 - d) * TWO_PI * sin(angle) + cos(angle));
					vert.norm[1] = -(amp / d) * y * ((1 - d) * TWO_PI * sin(angle) + cos(angle));
					vert.norm[2] = -1;

					d = 1.0f / sqrt(vert.norm[0] * vert.norm[0] + vert.norm[1] * vert.norm[1] + 1);
					vert.norm[0] *= d;
					vert.norm[1] *= d;
					vert.norm[2] *= d;
				}
			}
			for (i = 0; i < mesh_width; i++) {
				for (j = 0; j < mesh_height; j++) {
					face = get_face(frameNum, i, j);
					if ((((i / checker_size) & 1) ^ (j / checker_size) & 1) != 0) {
						face.color = check_color_0;
					} else {
						face.color = check_color_1;
					}
					pt1 = get_vert(frameNum, i, j).pos;
					pt2 = get_vert(frameNum, i, j + 1).pos;
					pt3 = get_vert(frameNum, i + 1, j + 1).pos;

					dp1[0] = pt2[0] - pt1[0];
					dp1[1] = pt2[1] - pt1[1];
					dp1[2] = pt2[2] - pt1[2];

					dp2[0] = pt3[0] - pt2[0];
					dp2[1] = pt3[1] - pt2[1];
					dp2[2] = pt3[2] - pt2[2];

					face.norm[0] = dp1[1] * dp2[2] - dp1[2] * dp2[1];
					face.norm[1] = dp1[2] * dp2[0] - dp1[0] * dp2[2];
					face.norm[2] = dp1[0] * dp2[1] - dp1[1] * dp2[0];

					d = 1.0f / sqrt(face.norm[0] * face.norm[0] + face.norm[1] * face.norm[1] + face.norm[2] * face.norm[2]);

					face.norm[0] *= d;
					face.norm[1] *= d;
					face.norm[2] *= d;
				}
			}
		}

	}

	@Override
	public void dbUpdate() {
		int x = mouseX();
		int y = mouseY();
		if (mouseLeftButtonActive()) {
			rotate_y = rotate_y + (y - yold) / 5.0f;
			rotate_x = rotate_x + (x - xold) / 5.0f;
			if (rotate_y > 90) {
				rotate_y = 90;
			}
			if (rotate_y < -90) {
				rotate_y = -90;
			}
		}
		if (mouseRightButtonActive()) {
			rotate_x = rotate_x + (x - xold) / 5.0f;
			translate_z = translate_z + (yold - y) / 50.0f;
			if (translate_z < 0.5f) {
				translate_z = 0.5f;
			}
			if (translate_z > 100) {
				translate_z = 100;
			}
		}
		xold = x;
		yold = y;
		cur_frame++;
	}

	@Override
	public void dbRender() {
		_vertex vert;
		_face face;
		float[] lastColor = null;
		float[] thisColor = null;

		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);

		cur_frame %= mesh.frames;

		ffp.ffpLoadIdentity();
		ffp.ffpTranslatef(0, 0, -translate_z);
		ffp.ffpRotatef(rotate_y, 1, 0, 0);
		ffp.ffpRotatef(rotate_x, 0, 1, 0);

		for (int i = 0; i < mesh.width; i++) {
			ffp.ffpBegin(FFP_QUAD_STRIP);
			lastColor = null;
			for (int j = 0; j < mesh.height; j++) {
				face = get_face(cur_frame, i, j);

				thisColor = face.color;
				ffp.ffpColor3fv(face.color);

				if (lastColor == null || (thisColor[0] != lastColor[0])) {
					if (lastColor != null) {
						ffp.ffpEnd();
						ffp.ffpBegin(FFP_QUAD_STRIP);
					}
					vert = get_vert(cur_frame, i, j);
					ffp.ffpNormal3fv(vert.norm);
					ffp.ffpVertex3fv(vert.pos);

					vert = get_vert(cur_frame, i + 1, j);
					ffp.ffpNormal3fv(vert.norm);
					ffp.ffpVertex3fv(vert.pos);
				}

				vert = get_vert(cur_frame, i, j + 1);
				ffp.ffpNormal3fv(vert.norm);
				ffp.ffpVertex3fv(vert.pos);

				vert = get_vert(cur_frame, i + 1, j + 1);
				ffp.ffpNormal3fv(vert.norm);
				ffp.ffpVertex3fv(vert.pos);

				lastColor = thisColor;
			}
			ffp.ffpEnd();
		}

		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
